/* aint_consts.h
 * definitions of constant values
 */

#ifndef AINT_CONSTS_H
#define AINT_CONSTS_H

// # of physical regs per thread
const int N_Physical_Regs = 2048; // must be power of 2

// # of logical regs per path
const int N_Logical_Regs = 32;

// Maximum file descriptors per process
const int MAX_FDNUM=33;

// Maximum processes estimate (for static arrays etc)
const int MAX_NPROCS = 256;
const int MAX_NTHREADS = 1024;
const int MAX_NPATHS = 16*1024;

// # of TSD keys per thread
const int AINT_pthread_keys_max = 255;

// Default stack size
const ulong KiloByte = (1<<10);
const ulong MegaByte = (1<<20);
const ulong GigaByte = (1<<30);

const ulong STACK_SIZE = 2*GigaByte;
const ulong HEAP_SIZE = 2*GigaByte;

const ulong SHMEM_SIZE = 2*GigaByte;


const ulong UNSP_SHMAT_START = ((ulong)(0x00010000));
const ulong UNSP_SHMAT_END = (UNSP_SHMAT_START + SHMEM_SIZE);

// size of the address space block affected by ldx_l and stx_c insts, in # of LSBs
const int LOCK_RANGE_SZ = 5;

// mask for converting addresses to quad-word aligned
const ulong QWORD_MASK = ~(7ul);


/* Align the simulated address space on an 8k boundary */
const ulong M_ALIGN = (8 * KiloByte);
const ulong PAGE_NUMBER_MASK = (~(M_ALIGN-1));
const ulong PAGE_OFFSET_MASK = (M_ALIGN-1);

// instruction flags
#define cdefine(a, b) const ulong a = b

cdefine(E_INSTR,    0x0);

cdefine(E_SPECIAL,  0x40);

cdefine(E_READ,     0x80);
cdefine(E_WRITE,    0x100);
cdefine(E_MEM_REF,  (E_READ|E_WRITE));

cdefine(E_SHARED,   0x200);

/* User cdefine(d) event type */
cdefine(E_USER,     0x400);

/* User cdefine(d) function */
cdefine(E_UFUNC,    0x800);

cdefine(E_FLOAT,    0x1000);

cdefine(E_BARRIER,  0x2000);

cdefine(E_LD_L,     0x4000);
cdefine(E_ST_C,    0x8000);
cdefine(E_LOCK,   (E_LD_L|E_ST_C));

cdefine(E_BSR,      1<<16);
cdefine(E_CONDBR,   1<<17);
cdefine(E_BR,       1<<18);
cdefine(E_JMP,      1<<19);
cdefine(E_JSR,      1<<20);
cdefine(E_RET,      1<<21);

cdefine(E_UNCONDBR, (E_BSR | E_BR | E_JMP | E_JSR | E_RET));
cdefine(E_BRANCH,   (E_UNCONDBR | E_CONDBR));

cdefine(E_CACHEOP,  1<<22);

cdefine(E_SYSCALL,  1<<23);
cdefine(E_PAL_OTHER, 1<<24);
cdefine(E_PAL, (E_SYSCALL | E_PAL_OTHER));
/* basic-block info */
/* FIXME: Do these have to be exclusive E_{events} values ? -Vega 07/16/96 */
cdefine(E_BB_FIRST, 1<<25);
cdefine(E_BB_LAST,  1<<26);

/* conditional mov insts - these read as well as possibly write dest reg */
cdefine(E_CMOV,     1<<27);

/*  sync insts: ldq_s and stq_s */
cdefine(E_SYNC,     1<<28);
/* the special types */
cdefine(E_FORK,  (E_SPECIAL | 1));
cdefine(E_M_FORK,  (E_SPECIAL | 2));
cdefine(E_YIELD,  (E_SPECIAL | 3));
cdefine(E_DONE,   (E_SPECIAL | 4));
cdefine(E_BLOCK,  (E_SPECIAL | 5));
cdefine(E_UNBLOCK, (E_SPECIAL | 6));
cdefine(E_LOCK_ATTEMPT, (E_SPECIAL | 7));
cdefine(E_LOCK_ACQUIRE, (E_SPECIAL | 8));
cdefine(E_LOCK_RELEASE, (E_SPECIAL | 9));
cdefine(E_BARRIER_ATTEMPT, (E_SPECIAL | 10));
cdefine(E_BARRIER_ACQUIRE, (E_SPECIAL | 11));
cdefine(E_PSEMA_ATTEMPT, (E_SPECIAL | 12));
cdefine(E_PSEMA_ACQUIRE, (E_SPECIAL | 13));
cdefine(E_VSEMA, (E_SPECIAL | 14));
cdefine(E_UNBLOCK_LIST, (E_SPECIAL | 15));
cdefine(E_WAIT, (E_SPECIAL | 16));
cdefine(E_EXIT, (E_SPECIAL | 17));
cdefine(E_FUNCTION, (E_SPECIAL | 18));
cdefine(E_SPROC, (E_SPECIAL | 19));
cdefine(E_KILL, (E_SPECIAL | 20));
cdefine(E_PREFORK, (E_SPECIAL | 21));
cdefine(E_MP_CREATE, (E_SPECIAL | 22));

#endif
