/* dllist.h
 * doubly-linked list interface & implementation
 * Stores a void pointer (generic type) in the data field
 */

#ifndef AINT_DLLIST_H
#define AINT_DLLIST_H


class dll_elem {
public:
  dll_elem *prev;
  dll_elem *next;
  void *data;

  // Make a new element, and insert after pred
  dll_elem(dll_elem *pred, void *value)
  {
    prev = pred;
    next = prev->next;
    prev->next = next->prev = this;
    data = value;
  }

  // make a new element as a header
  dll_elem()
  {
    prev = next = this;
    data = 0x0;
  }

  // When deleted, ensure consistent links between prev and next
  ~dll_elem()
  { prev->next = next; next->prev = prev; }

};


class dllist {
private:
  dll_elem *Head;
  dll_elem *Current;
public:
  dllist() { Head = new dll_elem(); Current = Head; }

  // Set Current to element of caller's liking
  void *dll_find(int (*Compare_Func)(void *))
  {
    for (Current = Head->next; Current != Head; Current=Current->next) {
      if ((*Compare_Func)(Current->data)) return Current->data;
    }
    return (void *) 0;
  }

  int dll_find_first(void *Compare_Value)
  {
    for (Current = Head->next; Current != Head; Current=Current->next) {
      if (Current->data == Compare_Value) return 1;
    }
    return 0;
  }

  void dll_insertbefore(void *Value)
  {  dll_elem *dummy = new dll_elem(Current->prev, Value); }

  void dll_insertafter(void *Value)
  {  dll_elem *dummy = new dll_elem(Current, Value); }

  // Using the dllist as a queue
  void dll_enqueue(void *Value)
  {  dll_elem *dummy = new dll_elem(Head->prev, Value); }

  void *dll_dequeue()
  {
    void *return_value = Head->next->data;
    if (Head->next != Head)
      delete Head->next;
    return return_value;
  }

  // the dllist as a circular buffer
  void dll_cb_add(void *Value) {
    dll_enqueue(Value);
    Current = Head->next;
  }

  void * dll_cb_next() {
    void * value = Current->data;
    Current = Current->next;
    if (Current == Head) Current = Current->next;
    return value;
  }

  int dll_cb_delete() {
    if (is_empty()) return 0;
    dll_elem *item = Current;
    Current = (Current->next == Head) ? Head->next : Current->next;
    delete item;
    return 1;
  }

  int dll_cb_locate(void *compare_value) {
    return dll_find_first(compare_value);
  }

  int dll_cb_delete(void *compare_value) {
    if (dll_find_first(compare_value)) {
      return dll_cb_delete();
    }
    return 0;
  }
    
  int is_empty()
  {  return (Head->next == Head); }

  ~dllist()
  {  delete Head; } // should be empty list

};


#endif
