/* instwin.h
 * the circular buffer implementation of the per-path inst window
 */


#ifndef AINT_INSTWIN_H
#define AINT_INSTWIN_H
// Size of the inflight-inst window from this path.
// This must be a power of 2.

const int INST_WIN_SIZE = 1024;

// Make this a template - circular Qstack
class InstWin {
public:
  /* InstWin(int _size) {  */
  InstWin() {
    // if (_size&(_size-1)) fatal("InstWin size must be a power of 2");
    // size = _size;
    size = INST_WIN_SIZE;
    cbif = new inflight_inst_ptr[size];
    head = tail = 0;
    count = 0;
  }

  ~InstWin() {
    delete [] cbif;
  }

  int enqueue(inflight_inst_ptr _ifi) {
    if (is_full()) return -1;
    count++;
    cbif[tail] = _ifi;
    _ifi->pos(tail);
    tail = SUCC_CIRC(tail, size);
    return 0;
  }
  

  int push(inflight_inst_ptr _ifi) {
    return enqueue(_ifi);
  }

  inflight_inst_ptr dequeue() {
    if (count == 0) return (inflight_inst_ptr) NULL;
    count--;
    inflight_inst_ptr ifi = cbif[head];
    cbif[head] = (inflight_inst_ptr) NULL;
    head = SUCC_CIRC(head, size);
    return ifi;
  }

  inflight_inst_ptr pop() {
    if (count == 0) return (inflight_inst_ptr) NULL;
    count--;
    tail = PRED_CIRC(tail, size);
    inflight_inst_ptr ifi = cbif[tail];
    cbif[tail] = (inflight_inst_ptr) NULL;
    return ifi;
  }

  int is_full() { return (count == size); }
  int is_empty() { return (count == 0); }

  inflight_inst_ptr peek(int n) { return cbif[n] ;}

  inflight_inst_ptr pred(inflight_inst_ptr _ifi) {
    return (tail==head) ? (inflight_inst_ptr) NULL :  cbif[PRED_CIRC(_ifi->pos(), size)];
  }

  inflight_inst_ptr succ(inflight_inst_ptr _ifi) {
    return (tail==head) ? (inflight_inst_ptr) NULL :  cbif[SUCC_CIRC(_ifi->pos(), size)];
  }


  inflight_inst_ptr *cbif;

  int head;
  int tail;
  int count;
  int size;
  
};


#endif
