/* Structures for describing each opcode */

#include <sys/types.h>
#include "icode.h"
#include "opcodes.h"
#include "newinst.h"

/* The instruction formats corresponding to the opcodes of instructions */
Iformat OpFormat[] = {
   PAL, /* Reserved, */ Memory,  Reserved, Reserved, Reserved, /* Reserved, */ Memory,  Reserved, Reserved,
   Memory, Memory, Reserved, Memory, Reserved, Reserved, Reserved, Memory,
   Operate, Operate, Operate, Operate, Reserved, Float, Float, Float,
   Memory, PAL, Memory, PAL, Reserved, PAL, PAL, PAL,
   Memory, Memory, Memory, Memory, Memory, Memory, Memory, Memory,
   Memory, Memory, Memory, Memory, Memory, Memory, Memory, Memory,
   Branch, Branch, Branch, Branch, Branch, Branch, Branch, Branch,
   Branch, Branch, Branch, Branch, Branch, Branch, Branch, Branch
  };

	
group_t pal_group[] = {
  {PAL_gentrap,	pal_gentrap},
  {PAL_rduniq,	pal_rduniq},
  {PAL_wruniq,	pal_wruniq},
  {PAL_bpt,	pal_bpt},
  {PAL_bugchk,	pal_bugchk},
  {PAL_chmk,	pal_chmk},
  {PAL_callsys,	pal_callsys},
  {PAL_imb,	pal_imb},
  {PAL_halt,	pal_halt},
  {PAL_draina,	pal_draina},
  {PAL_nphalt,	pal_nphalt},
  {PAL_cobratt,	pal_cobratt},
  {PAL_cserve ,	pal_cserve },
  {PAL_ipir,	pal_ipir},
  {PAL_cflush,	pal_cflush},
  {PAL_rti,	pal_rti},
  {PAL_rtsys,	pal_rtsys},
  {PAL_whami,	pal_whami},
  {PAL_rdusp,	pal_rdusp},
  {PAL_wrperfmon,	pal_wrperfmon},
  {PAL_wrusp,	pal_wrusp},
  {PAL_wrkgp,	pal_wrkgp},
  {PAL_rdps,	pal_rdps},
  {PAL_swpipl,	pal_swpipl},
  {PAL_wrent,	pal_wrent},
  {PAL_tbi,	pal_tbi},
  {PAL_rdval,	pal_rdval},
  {PAL_wrval,	pal_wrval},
  {PAL_swpctx,	pal_swpctx},
  {PAL_jtopal,	pal_jtopal},
  {PAL_wrvptptr,	pal_wrvptptr},
  {PAL_wrfen,	pal_wrfen},
  {PAL_mtpr_mces,	pal_mtpr_mces},
  {0, default_opnum}
};

group_t inta_group[] = {
  { inta_addl,	addl_opn },
  { inta_s4addl,	s4addl_opn },
  { inta_subl,	subl_opn },
  { inta_s4subl,	s4subl_opn },
  { inta_cmpbge,	cmpbge_opn },
  { inta_s8addl,	s8addl_opn },
  { inta_s8subl,	s8subl_opn },
  { inta_cmpult,	cmpult_opn },
  { inta_addq,	addq_opn },
  { inta_s4addq,	s4addq_opn },
  { inta_subq,	subq_opn },
  { inta_s4subq,	s4subq_opn },
  { inta_cmpeq,	cmpeq_opn },
  { inta_s8addq,	s8addq_opn },
  { inta_s8subq,	s8subq_opn },
  { inta_cmpule,	cmpule_opn },
  { inta_addlv,	addlv_opn },
  { inta_sublv,	sublv_opn },
  { inta_cmplt,	cmplt_opn },
  { inta_addqv,	addqv_opn },
  { inta_subqv,	subqv_opn },
  { inta_cmple,	cmple_opn },
  {0, default_opnum}
};

group_t intl_group[] = {
  { intl_and,	and_opn },
  { intl_bic,	bic_opn },
  { intl_cmovlbs,	cmovlbs_opn },
  { intl_cmovlbc,	cmovlbc_opn },
  { intl_bis,	bis_opn },
  { intl_cmoveq,	cmoveq_opn },
  { intl_cmovne,	cmovne_opn },
  { intl_ornot,	ornot_opn },
  { intl_xor,	xor_opn },
  { intl_cmovlt,	cmovlt_opn },
  { intl_cmovge,	cmovge_opn },
  { intl_eqv,	eqv_opn },
  { intl_cmovle,	cmovle_opn },
  { intl_cmovgt,	cmovgt_opn },
  {0, default_opnum}
};

group_t ints_group[] = {
  {ints_mskbl,	mskbl_opn},
  {ints_extbl,	extbl_opn},
  {ints_insbl,	insbl_opn},
  {ints_mskwl,	mskwl_opn},
  {ints_extwl,	extwl_opn},
  {ints_inswl,	inswl_opn},
  {ints_mskll,	mskll_opn},
  {ints_extll,	extll_opn},
  {ints_insll,	insll_opn},
  {ints_zap,	zap_opn},
  {ints_zapnot,	zapnot_opn},
  {ints_mskql,	mskql_opn},
  {ints_srl,	srl_opn},
  {ints_extql,	extql_opn},
  {ints_sll,	sll_opn},
  {ints_insql,	insql_opn},
  {ints_sra,	sra_opn},
  {ints_mskwh,	mskwh_opn},
  {ints_inswh,	inswh_opn},
  {ints_extwh,	extwh_opn},
  {ints_msklh,	msklh_opn},
  {ints_inslh,	inslh_opn},
  {ints_extlh,	extlh_opn},
  {ints_mskqh,	mskqh_opn},
  {ints_insqh,	insqh_opn},
  {ints_extqh,	extqh_opn},
  {0, default_opnum}
};

group_t fltv_group[] = {
  {fltv_addfc,	addfc_opn},
  {fltv_subfc,	subfc_opn},
  {fltv_mulfc,	mulfc_opn},
  {fltv_divfc,	divfc_opn},
  {fltv_cvtdgc,	cvtdgc_opn},
  {fltv_addgc,	addgc_opn},
  {fltv_subgc,	subgc_opn},
  {fltv_mulgc,	mulgc_opn},
  {fltv_divgc,	divgc_opn},
  {fltv_cvtgfc,	cvtgfc_opn},
  {fltv_cvtgdc,	cvtgdc_opn},
  {fltv_cvtgqc,	cvtgqc_opn},
  {fltv_cvtqfc,	cvtqfc_opn},
  {fltv_cvtqgc,	cvtqgc_opn},
  {fltv_addf,	addf_opn},
  {fltv_subf,	subf_opn},
  {fltv_mulf,	mulf_opn},
  {fltv_divf,	divf_opn},
  {fltv_cvtdg,	cvtdg_opn},
  {fltv_addg,	addg_opn},
  {fltv_subg,	subg_opn},
  {fltv_mulg,	mulg_opn},
  {fltv_divg,	divg_opn},
  {fltv_cmpgeq,	cmpgeq_opn},
  {fltv_cmpglt,	cmpglt_opn},
  {fltv_cmpgle,	cmpgle_opn},
  {fltv_cvtgf,	cvtgf_opn},
  {fltv_cvtgd,	cvtgd_opn},
  {fltv_cvtgq,	cvtgq_opn},
  {fltv_cvtqf,	cvtqf_opn},
  {fltv_cvtqg,	cvtqg_opn},
  {fltv_addfuc,	addfuc_opn},
  {fltv_subfuc,	subfuc_opn},
  {fltv_mulfuc,	mulfuc_opn},
  {fltv_divfuc,	divfuc_opn},
  {fltv_cvtdguc,	cvtdguc_opn},
  {fltv_addguc,	addguc_opn},
  {fltv_subguc,	subguc_opn},
  {fltv_mulguc,	mulguc_opn},
  {fltv_divguc,	divguc_opn},
  {fltv_cvtgfuc,	cvtgfuc_opn},
  {fltv_cvtgduc,	cvtgduc_opn},
  {fltv_cvtgqvc,	cvtgqvc_opn},
  {fltv_addfu,	addfu_opn},
  {fltv_subfu,	subfu_opn},
  {fltv_mulfu,	mulfu_opn},
  {fltv_divfu,	divfu_opn},
  {fltv_cvtdgu,	cvtdgu_opn},
  {fltv_addgu,	addgu_opn},
  {fltv_subgu,	subgu_opn},
  {fltv_mulgu,	mulgu_opn},
  {fltv_divgu,	divgu_opn},
  {fltv_cvtgfu,	cvtgfu_opn},
  {fltv_cvtgdu,	cvtgdu_opn},
  {fltv_cvtgqv,	cvtgqv_opn},
  {fltv_addfsc,	addfsc_opn},
  {fltv_subfsc,	subfsc_opn},
  {fltv_mulfsc,	mulfsc_opn},
  {fltv_divfsc,	divfsc_opn},
  {fltv_cvtdgsc,	cvtdgsc_opn},
  {fltv_addgsc,	addgsc_opn},
  {fltv_subgsc,	subgsc_opn},
  {fltv_mulgsc,	mulgsc_opn},
  {fltv_divgsc,	divgsc_opn},
  {fltv_cvtgfsc,	cvtgfsc_opn},
  {fltv_cvtgdsc,	cvtgdsc_opn},
  {fltv_cvtgqsc,	cvtgqsc_opn},
  {fltv_addfs,	addfs_opn},
  {fltv_subfs,	subfs_opn},
  {fltv_mulfs,	mulfs_opn},
  {fltv_divfs,	divfs_opn},
  {fltv_cvtdgs,	cvtdgs_opn},
  {fltv_addgs,	addgs_opn},
  {fltv_subgs,	subgs_opn},
  {fltv_mulgs,	mulgs_opn},
  {fltv_divgs,	divgs_opn},
  {fltv_cmpgeqs,	cmpgeqs_opn},
  {fltv_cmpglts,	cmpglts_opn},
  {fltv_cmpgles,	cmpgles_opn},
  {fltv_cvtgfs,	cvtgfs_opn},
  {fltv_cvtgds,	cvtgds_opn},
  {fltv_cvtgqs,	cvtgqs_opn},
  {fltv_addfsuc,	addfsuc_opn},
  {fltv_subfsuc,	subfsuc_opn},
  {fltv_mulfsuc,	mulfsuc_opn},
  {fltv_divfsuc,	divfsuc_opn},
  {fltv_cvtdgsuc,	cvtdgsuc_opn},
  {fltv_addgsuc,	addgsuc_opn},
  {fltv_subgsuc,	subgsuc_opn},
  {fltv_mulgsuc,	mulgsuc_opn},
  {fltv_divgsuc,	divgsuc_opn},
  {fltv_cvtgfsuc,	cvtgfsuc_opn},
  {fltv_cvtgdsuc,	cvtgdsuc_opn},
  {fltv_cvtgqsvc,	cvtgqsvc_opn},
  {fltv_addfsu,	addfsu_opn},
  {fltv_subfsu,	subfsu_opn},
  {fltv_mulfsu,	mulfsu_opn},
  {fltv_divfsu,	divfsu_opn},
  {fltv_cvtdgsu,	cvtdgsu_opn},
  {fltv_addgsu,	addgsu_opn},
  {fltv_subgsu,	subgsu_opn},
  {fltv_mulgsu,	mulgsu_opn},
  {fltv_divgsu,	divgsu_opn},
  {fltv_cvtgfsu,	cvtgfsu_opn},
  {fltv_cvtgdsu,	cvtgdsu_opn},
  {fltv_cvtgqsv,	cvtgqsv_opn},
  {0, default_opnum}
};

group_t flti_group[] = {
  {flti_addsc,	addsc_opn},
  {flti_subsc,	subsc_opn},
  {flti_mulsc,	mulsc_opn},
  {flti_divsc,	divsc_opn},
  {flti_sqrtsc, sqrtsc_opn},
  {flti_addtc,	addtc_opn},
  {flti_subtc,	subtc_opn},
  {flti_multc,	multc_opn},
  {flti_divtc,	divtc_opn},
  {flti_sqrttc, sqrttc_opn},
  {flti_cvttsc,	cvttsc_opn},
  {flti_cvttqc,	cvttqc_opn},
  {flti_cvtqsc,	cvtqsc_opn},
  {flti_cvtqtc,	cvtqtc_opn},
  {flti_addsm,	addsm_opn},
  {flti_subsm,	subsm_opn},
  {flti_mulsm,	mulsm_opn},
  {flti_divsm,	divsm_opn},
  {flti_sqrtsm, sqrtsm_opn},
  {flti_addtm,	addtm_opn},
  {flti_subtm,	subtm_opn},
  {flti_multm,	multm_opn},
  {flti_divtm,	divtm_opn},
  {flti_sqrttm, sqrttm_opn},
  {flti_cvttsm,	cvttsm_opn},
  {flti_cvttqm,	cvttqm_opn},
  {flti_cvtqsm,	cvtqsm_opn},
  {flti_cvtqtm,	cvtqtm_opn},
  {flti_adds,	adds_opn},
  {flti_subs,	subs_opn},
  {flti_muls,	muls_opn},
  {flti_divs,	divs_opn},
  {flti_sqrts, sqrts_opn},
  {flti_addt,	addt_opn},
  {flti_subt,	subt_opn},
  {flti_mult,	mult_opn},
  {flti_divt,	divt_opn},
  {flti_sqrtt, sqrtt_opn},
  {flti_cmptun,	cmptun_opn},
  {flti_cmpteq,	cmpteq_opn},
  {flti_cmptlt,	cmptlt_opn},
  {flti_cmptle,	cmptle_opn},
  {flti_cvtts,	cvtts_opn},
  {flti_cvttq,	cvttq_opn},
  {flti_cvtqs,	cvtqs_opn},
  {flti_cvtqt,	cvtqt_opn},
  {flti_addsd,	addsd_opn},
  {flti_subsd,	subsd_opn},
  {flti_mulsd,	mulsd_opn},
  {flti_divsd,	divsd_opn},
  {flti_sqrtsd, sqrtsd_opn},
  {flti_addtd,	addtd_opn},
  {flti_subtd,	subtd_opn},
  {flti_multd,	multd_opn},
  {flti_divtd,	divtd_opn},
  {flti_sqrttd, sqrttd_opn},
  {flti_cvttsd,	cvttsd_opn},
  {flti_cvttqd,	cvttqd_opn},
  {flti_cvtqsd,	cvtqsd_opn},
  {flti_cvtqtd,	cvtqtd_opn},
  {flti_addsuc,	addsuc_opn},
  {flti_subsuc,	subsuc_opn},
  {flti_mulsuc,	mulsuc_opn},
  {flti_divsuc,	divsuc_opn},
  {flti_sqrtsuc, sqrtsuc_opn},
  {flti_addtuc,	addtuc_opn},
  {flti_subtuc,	subtuc_opn},
  {flti_multuc,	multuc_opn},
  {flti_divtuc,	divtuc_opn},
  {flti_sqrttuc, sqrttuc_opn},
  {flti_cvttsuc,	cvttsuc_opn},
  {flti_cvttqvc,	cvttqvc_opn},
  {flti_addsum,	addsum_opn},
  {flti_subsum,	subsum_opn},
  {flti_mulsum,	mulsum_opn},
  {flti_divsum,	divsum_opn},
  {flti_sqrtsum, sqrtsum_opn},
  {flti_addtum,	addtum_opn},
  {flti_subtum,	subtum_opn},
  {flti_multum,	multum_opn},
  {flti_divtum,	divtum_opn},
  {flti_sqrttum, sqrttum_opn},
  {flti_cvttsum,	cvttsum_opn},
  {flti_cvttqvm,	cvttqvm_opn},
  {flti_addsu,	addsu_opn},
  {flti_subsu,	subsu_opn},
  {flti_mulsu,	mulsu_opn},
  {flti_divsu,	divsu_opn},
  {flti_sqrtsu, sqrtsu_opn},
  {flti_addtu,	addtu_opn},
  {flti_subtu,	subtu_opn},
  {flti_multu,	multu_opn},
  {flti_divtu,	divtu_opn},
  {flti_sqrttu, sqrttu_opn},
  {flti_cvttsu,	cvttsu_opn},
  {flti_cvttqv,	cvttqv_opn},
  {flti_addsud,	addsud_opn},
  {flti_subsud,	subsud_opn},
  {flti_mulsud,	mulsud_opn},
  {flti_divsud,	divsud_opn},
  {flti_sqrtsud, sqrtsud_opn},
  {flti_addtud,	addtud_opn},
  {flti_subtud,	subtud_opn},
  {flti_multud,	multud_opn},
  {flti_divtud,	divtud_opn},
  {flti_sqrttud, 	sqrttud_opn},
  {flti_cvttsud,	cvttsud_opn},
  {flti_cvttqvd,	cvttqvd_opn},
  {flti_cvtst,		cvtst_opn},
  {flti_cvtsts,		cvtsts_opn},
  {flti_addssuc,	addssuc_opn},
  {flti_subssuc,	subssuc_opn},
  {flti_mulssuc,	mulssuc_opn},
  {flti_divssuc,	divssuc_opn},
  {flti_sqrtssuc,       sqrtssuc_opn},
  {flti_addtsuc,	addtsuc_opn},
  {flti_subtsuc,	subtsuc_opn},
  {flti_multsuc,	multsuc_opn},
  {flti_divtsuc,	divtsuc_opn},
  {flti_sqrttsuc, 	sqrttsuc_opn},
  {flti_cvttssuc,	cvttssuc_opn},
  {flti_cvttqsvc,	cvttqsvc_opn},
  {flti_addssum,	addssum_opn},
  {flti_subssum,	subssum_opn},
  {flti_mulssum,	mulssum_opn},
  {flti_divssum,	divssum_opn},
  {flti_sqrtssum, 	sqrtssum_opn},
  {flti_addtsum,	addtsum_opn},
  {flti_subtsum,	subtsum_opn},
  {flti_multsum,	multsum_opn},
  {flti_divtsum,	divtsum_opn},
  {flti_sqrttsum, 	sqrttsum_opn},
  {flti_cvttssum,	cvttssum_opn},
  {flti_cvttqsvm,	cvttqsvm_opn},
  {flti_addssu,		addssu_opn},
  {flti_subssu,		subssu_opn},
  {flti_mulssu,		mulssu_opn},
  {flti_divssu,		divssu_opn},
  {flti_sqrtssu, 	sqrtssu_opn},
  {flti_addtsu,		addtsu_opn},
  {flti_subtsu,		subtsu_opn},
  {flti_multsu,		multsu_opn},
  {flti_divtsu,		divtsu_opn},
  {flti_sqrttsu, 	sqrttsu_opn},
  {flti_cmptunsu,	cmptunsu_opn},
  {flti_cmpteqsu,	cmpteqsu_opn},
  {flti_cmptltsu,	cmptltsu_opn},
  {flti_cmptlesu,	cmptlesu_opn},
  {flti_cvttssu,	cvttssu_opn},
  {flti_cvttqsv,	cvttqsv_opn},
  {flti_addssud,	addssud_opn},
  {flti_subssud,	subssud_opn},
  {flti_mulssud,	mulssud_opn},
  {flti_divssud,	divssud_opn},
  {flti_sqrtssud, 	sqrtssud_opn},
  {flti_addtsud,	addtsud_opn},
  {flti_subtsud,	subtsud_opn},
  {flti_multsud,	multsud_opn},
  {flti_divtsud,	divtsud_opn},
  {flti_sqrttsud, 	sqrttsud_opn},
  {flti_cvttssud,	cvttssud_opn},
  {flti_cvttqsvd,	cvttqsvd_opn},
  /*   {flti_cvttss,	cvttss_opn}, */
  {flti_addssuic,	addssuic_opn},
  {flti_subssuic,	subssuic_opn},
  {flti_mulssuic,	mulssuic_opn},
  {flti_divssuic,	divssuic_opn},
  {flti_sqrtssuic, 	sqrtssuic_opn},
  {flti_addtsuic,	addtsuic_opn},
  {flti_subtsuic,	subtsuic_opn},
  {flti_multsuic,	multsuic_opn},
  {flti_divtsuic,	divtsuic_opn},
  {flti_sqrttsuic, 	sqrttsuic_opn},
  {flti_cvttssuic,	cvttssuic_opn},
  {flti_cvttqsvic,	cvttqsvic_opn},
  {flti_cvtqssuic,	cvtqssuic_opn},
  {flti_cvtqtsuic,	cvtqtsuic_opn},
  {flti_addssuim,	addssuim_opn},
  {flti_subssuim,	subssuim_opn},
  {flti_mulssuim,	mulssuim_opn},
  {flti_divssuim,	divssuim_opn},
  {flti_sqrtssuim, 	sqrtssuim_opn},
  {flti_addtsuim,	addtsuim_opn},
  {flti_subtsuim,	subtsuim_opn},
  {flti_multsuim,	multsuim_opn},
  {flti_divtsuim,	divtsuim_opn},
  {flti_sqrttsuim, 	sqrttsuim_opn},
  {flti_cvttssuim,	cvttssuim_opn},
  {flti_cvttqsvim,	cvttqsvim_opn},
  {flti_cvtqssuim,	cvtqssuim_opn},
  {flti_cvtqtsuim,	cvtqtsuim_opn},
  {flti_addssui,	addssui_opn},
  {flti_subssui,	subssui_opn},
  {flti_mulssui,	mulssui_opn},
  {flti_divssui,	divssui_opn},
  {flti_sqrtssui, 	sqrtssui_opn},
  {flti_addtsui,	addtsui_opn},
  {flti_subtsui,	subtsui_opn},
  {flti_multsui,	multsui_opn},
  {flti_divtsui,	divtsui_opn},
  {flti_sqrttsui, 	sqrttsui_opn},
  {flti_cvttssui,	cvttssui_opn},
  {flti_cvttqsvi,	cvttqsvi_opn},
  {flti_cvtqssui,	cvtqssui_opn},
  {flti_cvtqtsui,	cvtqtsui_opn},
  {flti_addssuid,	addssuid_opn},
  {flti_subssuid,	subssuid_opn},
  {flti_mulssuid,	mulssuid_opn},
  {flti_divssuid,	divssuid_opn},
  {flti_sqrtssuid, 	sqrtssuid_opn},
  {flti_addtsuid,	addtsuid_opn},
  {flti_subtsuid,	subtsuid_opn},
  {flti_multsuid,	multsuid_opn},
  {flti_divtsuid,	divtsuid_opn},
  {flti_sqrttsuid, 	sqrttsuid_opn},
  {flti_cvttssuid,	cvttssuid_opn},
  {flti_cvttqsvid,	cvttqsvid_opn},
  {flti_cvtqssuid,	cvtqssuid_opn},
  {flti_cvtqtsuid,	cvtqtsuid_opn},
  {0, default_opnum}
};

group_t fltl_group[] = {
  {fltl_cvtlq,	cvtlq_opn},
  {fltl_cpys,	cpys_opn},
  {fltl_cpysn,	cpysn_opn},
  {fltl_cpyse,	cpyse_opn},
  {fltl_mt_fpcr,	mt_fpcr_opn},
  {fltl_mf_fpcr,	mf_fpcr_opn},
  {fltl_fcmoveq,	fcmoveq_opn},
  {fltl_fcmovne,	fcmovne_opn},
  {fltl_fcmovlt,	fcmovlt_opn},
  {fltl_fcmovge,	fcmovge_opn},
  {fltl_fcmovle,	fcmovle_opn},
  {fltl_fcmovgt,	fcmovgt_opn},
  {fltl_cvtql,	cvtql_opn},
  {fltl_cvtqlv,	cvtqlv_opn},
  {fltl_cvtqlsv,	cvtqlsv_opn},
  {0, default_opnum}
};

group_t misc_group[] = {
  {misc_trapb,	trapb_opn},
  {misc_excb,	excb_opn},
  {misc_mb,	mb_opn},
  {misc_wmb,	wmb_opn},
  {misc_fetch,	fetch_opn},
  {misc_fetch_m,	fetch_m_opn},
  {misc_rpcc,	rpcc_opn},
  {misc_rc,	rc_opn},
  {misc_rs,	rs_opn},
  {misc_ecb,    ecb_opn},
  {misc_wh64,   wh64_opn},
  {0, default_opnum}
};

group_t jsr_group[] = {
  {jsr_jmp,    jmp_opn},
  {jsr_jsr,    jsr_opn},
  {jsr_ret,    ret_opn},
  {jsr_jsr_coroutine, jsr_coroutine_opn},
  {0, default_opnum}
};

group_t intm_group[] = {
  {intm_mull,	mull_opn},
  {intm_mulq,	mulq_opn},
  {intm_umulh,	umulh_opn},
  {intm_mullv,	mullv_opn},
  {intm_mulqv,	mulqv_opn},
  {0, default_opnum}
};

group_t opc14_group[] = {
  {opc14_itofs, itofs_opn},
  {opc14_itoff, itoff_opn},
  {opc14_itoft, itoft_opn},  
  {0, default_opnum}
};

group_t opc1c_group[] = {
  {opc1c_perr, perr_opn},
  {opc1c_minub8, minub8_opn},
  {opc1c_minsb8, minsb8_opn},
  {opc1c_minuw4, minuw4_opn},
  {opc1c_minsw4, minsw4_opn},
  {opc1c_maxub8, maxub8_opn},
  {opc1c_maxsb8, maxsb8_opn},
  {opc1c_maxuw4, maxuw4_opn},
  {opc1c_maxsw4, maxsw4_opn},
  {opc1c_pklb, pklb_opn},
  {opc1c_pkwb, pkwb_opn},
  {opc1c_unpkbl, unpkbl_opn},
  {opc1c_unpkbw, unpkbw_opn},
  {opc1c_ctpop, ctpop_opn},
  {opc1c_ctlz, ctlz_opn},
  {opc1c_cttz, cttz_opn},
  {opc1c_ftois, ftois_opn},
  {opc1c_ftoit, ftoit_opn},
  {0, default_opnum}
};

group_t defgroup[] = {
  /* {op_opc01,	res_opc01}, */
  {op_ldq_s,    ldq_s_opn},

  {op_opc02,	res_opc02},
  {op_opc03,	res_opc03},
  {op_opc04,	res_opc04},

  /* {op_opc05,	res_opc05}, */
  {op_stq_s,    stq_s_opn},

  {op_opc06,	res_opc06},
  {op_opc07,	res_opc07},
  {op_lda,	lda_opn},
  {op_ldah,	ldah_opn},
  {op_opc0a,	ldb_opn},
  {op_ldq_u,	ldq_u_opn},
  {op_opc0c,	ldw_opn},
  {op_opc0d,	stw_opn},
  {op_opc0e,	stb_opn},
  {op_stq_u,	stq_u_opn},
  {op_opc14,	res_opc14},
  {op_pal19,	pal19_opn},
  {op_pal1b,	pal1b_opn},
  {op_opc1c,	res_opc1c},
  {op_pal1d,	pal1d_opn},
  {op_pal1e,	pal1e_opn},
  {op_pal1f,	pal1f_opn},
  {op_ldf,	ldf_opn},
  {op_ldg,	ldg_opn},
  {op_lds,	lds_opn},
  {op_ldt,	ldt_opn},
  {op_stf,	stf_opn},
  {op_stg,	stg_opn},
  {op_sts,	sts_opn},
  {op_stt,	stt_opn},
  {op_ldl,	ldl_opn},
  {op_ldq,	ldq_opn},
  {op_ldl_l,	ldl_l_opn},
  {op_ldq_l,	ldq_l_opn},
  {op_stl,	stl_opn},
  {op_stq,	stq_opn},
  {op_stl_c,	stl_c_opn},
  {op_stq_c,	stq_c_opn},
  {op_br,	br_opn},
  {op_fbeq,	fbeq_opn},
  {op_fblt,	fblt_opn},
  {op_fble,	fble_opn},
  {op_bsr,	bsr_opn},
  {op_fbne,	fbne_opn},
  {op_fbge,	fbge_opn},
  {op_fbgt,	fbgt_opn},
  {op_blbc,	blbc_opn},
  {op_beq,	beq_opn},
  {op_blt,	blt_opn},
  {op_ble,	ble_opn},
  {op_blbs,	blbs_opn},
  {op_bne,	bne_opn},
  {op_bge,	bge_opn},
  {op_bgt,	bgt_opn},
  {0, default_opnum}
};


/* desc_table[]: array of op_desc structs.
 * {char *opname, PFPI *func, int iflags }
 */

// FIXME: need to insert E_PAL flags in desc_table
struct op_desc desc_table[] = {
#include "desc_table.inc"
};

/* This is to force gcc to include exception information */
int mumble() {
  return 0;
}





