/* path.C
 * implementation of class path
 */

#include "aint.h"
#include "assert.h"

path::path(pathid_t my_id, path_ptr _parent, thread_ptr _pthread, aint_addr_t _pc)
{
  Logical_RegNum reg;

  pathid = my_id;
  parent = _parent;
  left = right = (path_ptr) NULL;
  pc = _pc;
  pthread = _pthread;
  instwin = new InstWin;
  priority = NULL;

  if (parent) {
    for (reg=0; reg < N_Logical_Regs; reg++) {
      RegMap[reg] = parent->RegMap[reg];
      RegMap[FP_REDIRECT_Z(reg)] = parent->RegMap[FP_REDIRECT_Z(reg)];
    }
  } else {
    assert(pathid == ROOT_PATH_ID);
    memcpy((void *)RegMap, (void *)pthread->RegMap, sizeof(RegMap));
  }

}

path::~path()
{ 
  delete instwin;
  if (pathid == ROOT_PATH_ID) {
    // Save regmap state in thread
    memcpy((void *) pthread->RegMap, (void *) RegMap, sizeof(RegMap));

  }
}


void
path::promote(pathid_t new_pathid)
{
  pathid = new_pathid;

  if (new_pathid == ROOT_PATH_ID) {
    parent = (path_ptr) NULL;
    pthread->path_tree = this;
  }

  if (left)
    left->promote(pathid*2);

  if (right)
    right->promote(pathid*2+1);

}
