/* thread.C
 * implementation of class thread
 */

#include <iostream.h>
#include <sys/types.h>

#include <aint_types.h>
#include <aint.h>
#include <globals.h>
#include <protos.h>

// Constructor - main thread in a new process
thread::thread(process_ptr myprocess)
{
  tid = ThreadCount++;

  Threads[tid] = this;

  pprocess = myprocess;

  for (int preg=0; preg < N_Physical_Regs; preg++) {
    Reg[preg].Int64 = 0x0;
  }

  // Logical Regs 0..31 are mapped to Phys Regs 0..31; Log FP 0..30 mapped to Phys 32..62; FP31 to 31.
  Reset_InUse();

  for (Logical_RegNum reg=0; reg < N_Logical_Regs; reg++) {
    RegMap[reg] = (Physical_RegNum) reg;
    Set_InUse(RegMap[reg]);
    RegMap[FP_REDIRECT_Z(reg)] = (Physical_RegNum) FP_REDIRECT_Z(reg);
    Set_InUse(RegMap[FP_REDIRECT_Z(reg)]);
  }

  // At this point, Physical Regs 0..62 are in use
  { // Initializethe free register stack: push regs 63 and up.
    FreeRegStackTop = -1;
    Physical_RegNum pr;
    for (pr=63; pr<N_Physical_Regs; pr++) {
      Clr_InUse(pr); // Also pushes pr on the FreeRegStack
    }
  }

  FetchCount = 0;
  picode = (icode_ptr) NULL;
  unique = 0l;

  locked_addr = (aint_addr_t) 0x0;
  lock_flag = 0;

  runstate = R_RUN;
  exitcode = 0;
  next_thread = (thread_ptr) NULL;

  path_tree = (path_ptr) NULL;
  // path_tree = new path(ROOT_PATH_ID, (path_ptr NULL), this, pprocess->entry_point);

  bzero((char *)TSD, AINT_pthread_keys_max*sizeof(ulong));

  sim_thread_begin(tid, pprocess->entry_point);

  StoreBuffer = new Store_Buffer;

}


// Constructor: main thread in newly forked process
thread::thread(process_ptr parent, thread_ptr parent_thread)
{
  tid = ThreadCount++;
  Threads[tid] = this;
  pprocess = parent->youngest;

  FetchCount = 0;

  StoreBuffer = new Store_Buffer;

  fatal("FIXME: thread constructor for fork() is incomplete. Need to copy thread state..." << endl);

  bzero((char *)TSD, AINT_pthread_keys_max*sizeof(ulong));

  // path_tree = new path(ROOT_PATH_ID, (path_ptr NULL), this, pprocess->entry_point);
}


/* Constructor: new thread created using pthread_create
 * start_pc is used only to pass on to the back-end
 */

thread::thread(thread_ptr pthread, aint_addr_t start_pc)
{
  tid = ThreadCount++;
  Threads[tid] = this;
  pprocess = pthread->pprocess;

  FetchCount = 0;

  StoreBuffer = new Store_Buffer;


  // Allocate stack space
  ulong stack_end = pprocess->allocate_thread_stack(tid);

  FreeRegStackTop = -1;
  for (int preg=0; preg < N_Physical_Regs; preg++) {
    // Copy reg in-use information. This could be done more efficiently, but in a less intuitive manner
    // by copying the InUse[] array directly
    if (pthread->Is_InUse(preg)) {
      Set_InUse(preg);
    } else {
      Clr_InUse(preg);
    }

    // Copy the physical register state
    Reg[preg].Int64 = pthread->Reg[preg].Int64;
  }

  // Copy the reg map
  for (int lreg=0; lreg<2*N_Logical_Regs; lreg++) {
    RegMap[lreg] = pthread->path_tree->RegMap[lreg];
  }

  // replace the stack pointer to point to the new stack
  Reg[RegMap[SP]].Int64 = stack_end;

  FetchCount = 0;
  picode = (icode_ptr) NULL;
  unique = 0l;

  locked_addr = (aint_addr_t) 0x0;
  lock_flag = 0;

  runstate = R_RUN;
  exitcode = 0;
  next_thread = (thread_ptr) NULL;

  path_tree = (path_ptr) NULL;
  
  sim_thread_begin(tid, start_pc);

  bzero((char *)TSD, AINT_pthread_keys_max*sizeof(ulong));

  StoreBuffer = new Store_Buffer;
}
