#include <stdio.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include "alpha_lock.h"


int main(int argc, char *argv[])
{
  /* Allocate shmem, fork 2 processes, and do several thousand locked reads
   * and writes. Wait for child to die, then print results
   */
  key_t k;
  long shmid;
  int i, n, s, pid;
  int *seg, x, mysum;
  long tmp;
  char whoami[512];

  struct shmid_ds buf;

  fprintf(stderr,"Usage: locks <tests> <seed>\n");

  sscanf(argv[1], "%d", &n);
  sscanf(argv[2], "%d", &s);
  srandom(s);

  k = ftok("/vmunix", getpid());

  shmid = shmget(k, sizeof(int)*4096, IPC_CREAT|0666);

  pid = fork();

  if (pid == 0) {
    sprintf(whoami,"child[%d]", getpid());
  } else {
    sprintf(whoami,"parent[%d]", getpid());
  }

  seg = shmat(shmid, 0, 0);
  if (seg <= 0){
    perror("shmat");
    exit(1);
  } else {
    fprintf(stderr,"%s: Segment address is %lx\n",
	    whoami, (long) seg);
  }

#if 0
  asm("_again: ldl_l %0 %1; xor %0 %0 %0; stl_c %0 %1; beq %0, _again" :
      "=r" (x), "=o"(*seg));
  alpha_lock(seg);
#else
#endif

  /* Have to do a synchronization here: need a barrier. If parent, wait until
   * child sets seg[1] to 1. If child, set seg[1] to 1 */

  if (pid) {
    printf("%s : Parent - begining wait loop\n", whoami); fflush(stdout);
    alpha_mb();
    while(seg[1] != 34) {
      alpha_mb();
    }
    printf("%s : Parent - end of wait loop\n", whoami); fflush(stdout);
  } 
  else {
    seg[0] = 50;
    seg[1] = 34;
    alpha_mb();
    printf("%s: Child - succeeded in zeroing mem loc.\n", whoami); fflush(stdout);
    alpha_mb();
  }

  mysum = 0;
  
  printf("%s : Begining long sequence of additions.\n", whoami);

  /* Do a long sequence of additions. */
  for(i=0; i<n; i++) {
/*     tmp = random()>>24; */
    tmp = getpid()+ 5;
    mysum += tmp;
    /* Do an atomic update */
#if 0
    asm("_again2: ldl_l %0 %1; addl %0 %2 %0; stl_c %0 %1; beq %0, _again2":
	"&=r" (x), "=o"(*seg): "r"(tmp));
#else
    alpha_atomic_add(seg, tmp);
#endif
    alpha_mb();
  }
  printf("%s: mysum %d, shared sum %d\n", whoami, mysum, *seg);
  /* Detach mem seg */
  shmdt(seg);

  wait(&tmp);

  printf("parent?: shared sum is %d\n", *seg);
  shmctl(shmid, IPC_RMID, &buf);
}
